@@ -512,6 +512,14 @@ static int
 compatible_formats(CARD8 op, PicturePtr dst, PicturePtr src)
 {
     if (op == PictOpSrc) {
+        /* We can't do direct copies between different depths at 16bpp
+         * because r,g,b are allocated to different bits.
+         */
+        if (dst->pDrawable->bitsPerPixel == 16 &&
+            dst->pDrawable->depth != src->pDrawable->depth) {
+            return 0;
+        }
+
         if (src->format == dst->format)
             return 1;
 
@@ -1279,12 +1287,17 @@ glamor_convert_gradient_picture(ScreenPtr screen,
     PixmapPtr pixmap;
     PicturePtr dst = NULL;
     int error;
+    PictFormatPtr pFormat;
     PictFormatShort format;
 
-    if (!source->pDrawable)
+    if (source->pDrawable) {
+        pFormat = source->pFormat;
+        format = pFormat->format;
+    } else {
         format = PICT_a8r8g8b8;
-    else
-        format = source->format;
+        pFormat = PictureMatchFormat(screen, 32, format);
+    }
+
 #ifdef GLAMOR_GRADIENT_SHADER
     if (!source->pDrawable) {
         if (source->pSourcePict->type == SourcePictTypeLinear) {
@@ -1320,10 +1333,7 @@ glamor_convert_gradient_picture(ScreenPtr screen,
         return NULL;
 
     dst = CreatePicture(0,
-                        &pixmap->drawable,
-                        PictureMatchFormat(screen,
-                                           PIXMAN_FORMAT_DEPTH(format),
-                                           format), 0, 0, serverClient, &error);
+                        &pixmap->drawable, pFormat, 0, 0, serverClient, &error);
     glamor_destroy_pixmap(pixmap);
     if (!dst)
         return NULL;
